<?php
class Grup extends DomainObject {
    /** @var $projecte Projecte */
	protected $projecte;
    /** @var $perfilsContractacio GrupPerfilContractacio[] */
	protected $perfilsContractacio;
    /** @var $empresesPractiques Empresa[] */
	protected $empresesPractiques;
    /** @var $poblacioCataleg Poblacio */
	protected $poblacioCataleg;
    /** @var $empresaCataleg Empresa */
	protected $empresaCataleg;
	protected $requisitsDocs;
	protected $requisitsCamps;
    /** @var $modalitat TipusModalitatGrup */
	protected $modalitat;
    /** @var $estat TipusEstatGrup */
	protected $estat;
    /** @var $etiquetes TipusEtiquetaAccio[] */
    protected $etiquetes;
    /** @var $grupModuls GrupModul[] */
    protected $grupModuls;

    /**
     * @param Persona $persona
     * @return bool
     */
    function comprovaAlta(Persona $persona)
    {
        $resultat = false;
        $dataAlta = '';
        $dataBaixa = '';
        $mpg = new PersonaGrupMapper();
        $apunts = $mpg->findBy(array('idPersona' => $persona->getId(), 'idGrup' => $this->getId()));
        foreach ($apunts as $apunt) {
            if ($apunt->esAlta()) {
                $dataAlta = $apunt->getField('data');
                $resultat = true;
            } elseif ($apunt->esBaixa()) {
                $dataBaixa = $apunt->getField('data');
                $resultat = false;
            }
        }
        //return $dataAlta > $dataBaixa;
        return $resultat;
    }

    /**
     * @return string
     */
    function getKeyName() {
		return 'idGrup';
	}

    /**
     * @return null|string
     */
    function getNomOficial()
    {
        $nom = $this->getField('nomOficial');
        if (!empty($nom)) {
            return $nom;
        } else {
            return $this->getNom();
        }

    }

    /**
     * @return null|Projecte
     */
    function getProjecte() {
		if (!isset($this->projecte)) {
			$m = new ProjecteMapper();
			$this->projecte = $m->find($this->dades['idProjecte']);
		}
		return $this->projecte;
	}

    /**
     * @return null|Poblacio
     */
    function getPoblacioCataleg(){
		if (!isset($this->poblacioCataleg)) {
			$m = new PoblacioMapper();
			$this->poblacioCataleg = $m->find($this->dades['idPoblacioCataleg']);
		}
		return $this->poblacioCataleg;
	}

    /**
     * @return Empresa|null
     */
    function getEmpresaCataleg(){
		if (!isset($this->empresaCataleg)) {
			$m = new EmpresaMapper();
			$this->empresaCataleg = $m->find($this->dades['idEmpresaCataleg']);
		}
		return $this->empresaCataleg;
	}

	function getDataInici() {
		return $this->getDateField('dataInici');
	}

	function getDataFi() {
		return $this->getDateField('dataFi');
	}

    /**
     * @return float|null
     */
    function getHores() {
		return $this->getField('totalHores');
	}

    /**
     * @return Empresa[]
     */
    function getEmpresesPractiques() {
		if (!isset($this->empresesPractiques)) {
			$m = new EmpresaMapper();
			$sql = "SELECT idEmpresa FROM empresesGrups WHERE idGrup=".$this->getId()." ";
			$rs = DB::query($sql);
			while ($dd=mysql_fetch_assoc($rs)) {
				if ($foo = $m->find($dd['idEmpresa'])) {
					$this->empresesPractiques[] = $foo;
				}
			}

		}
		return $this->empresesPractiques;
	}

	/**
	 * Perfils Contractacio
	 * @return GrupPerfilContractacio[]
	 */
	function getPerfilsContractacio() {
		if (!isset($this->perfilsContractacio)) {
			$m = new GrupPerfilContractacioMapper();
			$sql = "SELECT id FROM grupsPerfilsContractacio  WHERE idGrup={$this->dades['idGrup']} ";
			$rs = DB::query($sql);
			while ($dd=mysql_fetch_assoc($rs)) {
				$this->perfilsContractacio[] = $m->find($dd['id']);
			}

		}
		return $this->perfilsContractacio;
	}

    /**
     * @return int
     */
    function getPlacesDisponibles(){
		if($this->dades['numPlaces']>0){
			$m = new PersonaGrupMapper();
			$ops=array($tipusAccio=>1,$idGrup=>$this->dades['idGrup']);
			$inscripcionsGrup=$m->findBy($ops);
			return ($this->dades['numPlaces']-count($inscripcionsGrup));
		}else{
			return 99;
		}
	}

    /**
     * @param $p Persona
     * @return bool|Requisits
     */
    function requisitsPendentsInscripcio(Persona $p){
		$requisitsNoCompleix = false;
		
		if(!isset($this->requisitsDocs)){
			$m = new GrupTipusDocPersonaMapper();
			$ops = array('idGrup'=>$this->dades['idGrup'],'esWeb'=>$this->dades['esWeb']);
			$this->requisitsDocs=$m->findBy($ops);
			foreach ($this->requisitsDocs as $requisit){
				if(!$p->comprovaDocument($requisit->getIdTipusDocPersona())){
					$requisitsNoCompleix[]=array($requisit->getIdTipusDocPersona(),2);
				}
			}
		}
		
		if(!isset($this->requisitsCamps)){
			$m = new GrupWebCampsMapper();
			$webCamps = $m->find($this->dades['idGrup']);
			if(!is_object($webCamps)){
				return false;
			}
			$camps = $webCamps->getCamps();
			//unset($camps->dades['idGrup']);
			foreach($camps as $camp){
					if($camp=='FormacioReglada'){
						$formacio=$p->getFormacioReglada();
						if(empty($formacio)){
							$requisitsNoCompleix[]=array($camp,1);
						}
					}elseif(($p->getField($camp)=='')||($p->getField($camp)=='0000-00-00')){
						$requisitsNoCompleix[]=array($camp,1);
					}
			}
		}
		
		if(!empty($requisitsNoCompleix)){
			return new Requisits($requisitsNoCompleix);
		}else{
			return false;
		}
	}

    /**
     * @return null|TipusModalitatGrup
     */
    function getModalitat(){
		if(!isset($this->modalitat)){
			$m=new TipusModalitatGrupMapper();
			$this->modalitat=$m->find($this->dades['idTipusModalitat']);
		}
		return $this->modalitat;
	}

    /**
     * @return null|TipusEstatGrup
     */
    function getEstat(){
		if(!isset($this->estat)){
			$m=new TipusEstatGrupMapper();
			$this->estat=$m->find($this->dades['idEstatGrup']);
		}
		return $this->estat;
	}

	
	function setdataInici($v) {
		$parts = explode('/', $v);
		$data = date('Y-m-d', mktime(0,0,0,$parts[1],$parts[0],$parts[2]));
		$this->setField('dataInici', $data);
	}
	
	function setdataFi($v) {
		$parts = explode('/', $v);
		$data = date('Y-m-d', mktime(0,0,0,$parts[1],$parts[0],$parts[2]));
		$this->setField('dataFi', $data);
	}

    /**
     * @return array|TipusEtiquetaAccio[]
     */
    function getEtiquetes()
    {
        if (!isset($this->etiquetes)) {
            $m = new TipusEtiquetaAccioMapper();
            return $m->findBy(array('idGrup' => $this->getId()));
        }
        return $this->etiquetes;
    }

    /**
     * @return TipusEtiquetaAccio[]
     */
    function getEtiquetesWeb()
    {
        /** @var $etiquetes TipusEtiquetaAccio[] */
        $etiquetes = array();
        $tmp = $this->getEtiquetes();
        foreach ($tmp as $etiqueta) {
            if ($etiqueta->getField('esSurtAlWeb')) {
                $etiquetes[] = $etiqueta;
            }
        }
        return $etiquetes;
    }

    /**
     * @param TipusEtiquetaAccio $etiqueta
     * @return bool
     */
    function teEtiqueta(TipusEtiquetaAccio $etiqueta) {
        foreach ($this->getEtiquetes() as $etiqueta2) {
            if ($etiqueta==$etiqueta2) {
                return true;
            }
        }
        return false;
    }

    /**
     * @return GrupModul[]
     */
    function getModuls() {
        if (!isset($this->grupModuls)) {
            $m = new GrupModulMapper();
            $this->grupModuls = $m->findBy(array('idGrup'=>$this->getId()));
        }
        return $this->grupModuls;
    }
}


