<?php
class QuestionariResultat extends DomainObject {
    /** @var $persona Persona */
	protected $persona;
    /** @var $empresa Empresa */
	protected $empresa;
    /** @var $grup Grup */
	protected $grup;
    /** @var $oferta Oferta */
	protected $oferta;
    /** @var $questionari Questionari */
	protected $questionari;
    /** @var $usuariAssigna Usuari */
	protected $usuariAssigna;
    /** @var $respostes QuestionariResposta[] */
	protected $respostes;
    /** @var $resultatsCategories QuestionariResultatCategoria[] */
    protected $resultatsCategories;

	function addWeb(array $dd) {
		//generem token
		$this->setField('token', uniqid());
		//posem la data d'assignació
		$this->setField('dataAssigna', date('Y-m-d'));

		parent::addWeb($dd);

		//mirem si hi ha plantilla d'enviament per enviar correu
		if (is_numeric(CFG::PanellControl('idPlantillaMailQuest'))) {
			if ($mailer =& JFactory::getMailer()) {
				//Obtenim la Plantilla, la Persona i el Curs
				$plm = new PlantillaMailMapper();
				$pl = $plm->find(CFG::PanellControl('idPlantillaMailQuest'));

				$c = $this->getGrup();
				$p = $this->getPersona();

				//FIXME l'enviament nomès funciona amb Joomla
				$mailer->isHTML(true);
				$mailer->SetLanguage("es");
				$mailer->setSender(array($pl->getField('emailRemitent'),$pl->getField('remitent')));
				//	$mailer->AddAddress($p->getEmail(),$p->getNom());
				$mailer->AddAddress('suport@spinvilanova.com',$p->getNom());

				//Formategem l'assumpte
				$subject = str_replace('$nomCurs',$c->getNom(),$pl->getField('assumpte'));
				$mailer->setSubject($subject);

				$dadesMail = array('$nomCurs'=>$c->getNom(),'$nomPersona'=>$p->getNomComplert(),'$linkQuestionari'=>CFG::PanellControl('webQuestionaris').'?token='.$this->getField('token'),'$dataLimit'=>$this->getDateField('dataLimit'));
				$mailer->setBody($pl->getMail($dadesMail));
				$dadesApunt=array('idGrup'=>$c->getId(),'comentaris'=>$subject);
				$p->apuntAuto('comunicacioMail',$dadesApunt);
				$send =& $mailer->Send();
			}
		}
	}

	function getKeyName() {
		return 'id';
	}

	function getNom() {
		return $this->dades['token'];
	}

	function getDataInici() {
		return $this->getDatetimeField('dataInici');
	}

	function getDataFi() {
		return $this->getDatetimeField('dataFi');
	}

	function getDataAssigna() {
		return $this->getDataField('dataAssigna');
	}

    /**
     * @return null|Questionari
     */
    function getQuestionari() {
		if (!isset($this->questionari)) {
			$m = new QuestionariMapper();
			$this->questionari = $m->find($this->dades['idQuestionari']);
		}
		return $this->questionari;
	}

    /**
     * @return QuestionariResultatResposta[]
     */
    function getRespostes() {
		if (!isset($this->respostes)) {
			$m = new QuestionariResultatRespostaMapper();
			$this->respostes = $m->findBy(array('idQuestionariResultat'=>$this->getId()));
		}
		return $this->respostes;
	}

    /**
     * @param $pregunta QuestionariPregunta
     * @return bool|QuestionariResposta
     */
    function getResposta(QuestionariPregunta $pregunta) {
		if (is_a($pregunta, 'QuestionariPregunta')) {
			$rs = $this->getRespostes();
			foreach ($rs as $r) {
				if ($r->getField('idPregunta') == $pregunta->getId()) {
					return $r;
				}
			}
		}
		return false;
	}

    /**
     * @return null|Persona
     */
    function getPersona() {
		if (!isset($this->persona)) {
			$m = new PersonaMapper();
			$this->persona = $m->find($this->dades['idPersona']);
		}
		return $this->persona;
	}

    /**
     * @return Empresa|null
     */
    function getEmpresa() {
		if (!isset($this->empresa)) {
			$m = new EmpresaMapper();
			$this->empresa = $m->find($this->dades['idEmpresa']);
		}
		return $this->empresa;
	}

    /**
     * @return null|Oferta
     */
    function getOferta() {
		if (!isset($this->oferta)) {
			$m = new OfertaMapper();
			$this->oferta = $m->find($this->dades['idOferta']);
		}
		return $this->oferta;
	}

    /**
     * @return Grup|null
     */
    function getGrup() {
		if (!isset($this->grup)) {
			$m = new GrupMapper();
			$this->grup = $m->find($this->dades['idGrup']);
		}
		return $this->grup;
	}

    /**
     * @return array|QuestionariResultatCategoria[]
     */
    function getResultatsCategories() {
        if (!isset($this->resultatsCategories)) {
            $m = new QuestionariResultatCategoriaMapper();
            $this->resultatsCategories = $m->findBy(array('idQuestionariResultat'=>$this->getId()));
        }
        return $this->resultatsCategories;

    }
    /**
     * Fianlitza el questionari
     */
    function finalitza() {
		if (!strtotime($this->getField('dataFi'))) {
			//comprovem que no s'ha finalitzat per no repetir la finalització
			$this->setField('dataFi', date('Y-m-d H:i:s'));
			$this->getMapper()->update($this);
			if ($persona = $this->getPersona()) {
				if ($grup = $this->getGrup()) {
					$dd['idGrup']=$grup->getId();
				}
				$persona->apuntAuto('fiQuestionariPersona', $dd);
                $this->finalitzaAltes();
			}
		}
	}

    /**
     * Dona per iniciat l'assignació de qüestionari
     */
    function inicia() {
        if (!strtotime($this->getField('dataInici'))) {
            $this->setField('dataInici', date('Y-m-d H:i:s'));
            $this->getMapper()->update($this);
            if ($persona = $this->getPersona()) {
                if ($grup = $this->getGrup()) {
                    $dd['idGrup']=$grup->getId();
                    $persona->apuntAuto('iniciQuestionariPersona', $dd);
                }
            }
            $this->updateWeb(array());
        }
    }

    /**
     * @return bool
     */
    function esIniciat() {
        if (is_null($this->getDataInici())) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * @return bool
     */
    function esFinalitzat() {
        if (is_null($this->getDataFi())) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Genera les inscripcions en funció dels barems per la finalització d'un qüestionari
     */
    private function finalitzaAltes() {
        $resultats = $this->getResultatsCategories();
        foreach ($resultats as $resultat) {
            $categoria = $resultat->getCategoria();
            if ($barem = $categoria->calcBarem($resultat->getField('puntuacio'))) {
                if ($curs = $barem->getCurs()) {
                 $curs->inscriuPersona($this->getPersona());
                }
            }
        }
    }

    /**
     * @return QuestionariPregunta
     */
    function preguntaSeguent() {
        if ($this->esIniciat()) {
            $respostes = $this->getRespostes();
            if ($this->getQuestionari()->getField('esOrdreAleatori')) {
                $preguntes = $this->getQuestionari()->getPreguntes();
                shuffle($preguntes);
                foreach ($preguntes as $preguta) {
                    $trobat = false;
                    foreach ($respostes as $resposta) {
                        if ($resposta->getPregunta() == $preguta) {
                            $trobat = true;
                            break;
                        }
                    }
                    if (!$trobat) return $preguta;
                }
            } elseif ($resposta = $respostes[count($respostes) - 1]->getResposta()) {
                return $resposta->getPreguntaSeguent();
            } else {
                return $respostes[count($respostes) - 1]->getPregunta()->getPreguntaSeguent();
            }
        } else {
            return $this->preguntaInici();
        }
    }

    /**
     * @return QuestionariPregunta
     */
    function preguntaInici() {
        if ($this->getQuestionari()->getField('esOrdreAleatori')) {
            $preguntes = $this->getQuestionari()->getPreguntes();
            shuffle($preguntes);
            return $preguntes[0];
        } else {
            return $this->getQuestionari()->getPreguntaInici();
        }
    }

    /**
     * Temps total emprat en segons
     * @return float
     */
    function tempsEmprat()
    {
        $temps = 0;
        $respostes = $this->getRespostes();
        foreach ($respostes as $resposta) {
            $temps += floatval($resposta->getField('temps'));
        }
        return $temps;
    }

    /**
     * Total temps disponible
     * @return float
     */
    function tempsDisponible() {
        return floatval($this->getQuestionari()->getField('temps')-$this->tempsEmprat());
    }
}

