<?php

abstract class DomainObject {
	protected $dades;
	protected $dadesDirty; //camps pendents d'actualitzar

	function __construct(array $array=array()) {
		if ( count($array)==0 || is_null( $array[$this->getKeyName()] ) ) {
			$this->markNew();
		}
		$m = $this->getMapper();
		$camps = DB::tableFields($m->targetTable());
		$this->dades = array_fill_keys($camps, '');
		$this->dades = array_merge($this->dades, array_map('stripslashes', $array));
		$this->dadesDirty = array();
	}

	function markNew() {
		ObjectWatcher::addNew( $this );
	}

	function markDeleted() {
		ObjectWatcher::addDelete( $this );
	}

	function markDirty() {
		ObjectWatcher::addDirty( $this );
	}

	function markClean() {
		//		unset($this->dadesDirty);
		ObjectWatcher::addClean( $this );
	}

	function __toString() {
		return ''.$this->getNom();
	}

    /**
     * @return string
     */
    function getNom() {
		return $this->dades['nom'];
	}

    /**
     * @param $field string
     * @return null|mixed
     */
    function getField($field) {
		if (key_exists($field, $this->dades)) {
			return $this->dades[$field];
		} else {
			return null;
		}
	}

    /**
     * @param $field
     * @return null|string
     */
    function getDateField($field) {
		if (empty($this->dades[$field]) || $this->dades[$field]=='0000-00-00') return null;
		if ($data = strtotime($this->dades[$field])) {
			return date('d/m/Y', $data);
		} else {
			return null;
		}
	}

	function getDatetimeField($field) {
		if (empty($this->dades[$field]) || $this->dades[$field]=='0000-00-00 00:00:00') return null;
		if ($data = strtotime($this->dades[$field])) {
			return date('H:i:s d/m/Y', $data);
		} else {
			return null;
		}
	}


	function getDirtyFields() {
		return array_intersect_key($this->dades, array_flip($this->dadesDirty));
	}

	function setField($field, $value) {
		$this->dades[$field]=$value;
		if (!in_array($field, $this->dadesDirty)) {
			$this->dadesDirty[]=$field;
		}
		$this->markDirty();
	}

	function _apuntAutoModWeb() {
		//Si tenim una referència a una persona hem de fer un apunt de modificació web
		$idPersona = $this->getField('idPersona');
		if ($idPersona>0) {
			if (is_a($this, 'Persona')) {
				$persona = $this;
			} else {
				$m = new PersonaMapper();
				$persona = $m->find($idPersona);
			}
			if ($ap = $persona->apuntAuto('modDadesWeb', $this->dades)) {
				return $ap->getId();
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

	function updateWeb(array $dd) {
		$ida = $this->_apuntAutoModWeb();
		//Mirem totes les dades que ens envien
		foreach ($dd as $k=>$v) {
			//Comprovem si la dada correspon amb una propietat de l'objecte
			if (key_exists($k, $this->dades)) {
				//Nomès actualitzem si el valor és diferent
				$vAnt = $this->getField($k);
				//FIXME Problema amb el format de les dates
				if (count(explode('-', $vAnt))==3 && $d=strtotime($vAnt)) $vAnt = date('d/m/Y', $d);
				if ($vAnt!=$v) {
					//FIXME escriure log a personesCanvisWeb més elaborat
					if (is_numeric($this->getField('idPersona'))) {
						$sql = sprintf("INSERT INTO personesCanvisWeb
								(idPersona, idPersonaGrup, idK, idV, pestanya, accio, camp, valorAntic, valor, data, esSinc, esVerificat )
								VALUES
								(%d, '%d', '%s', %d, '%s', 'M', '%s', '%s', '%s', CURDATE(), 1, 0) ", $this->getField('idPersona'), $ida, $this->getKeyName(), $this->getId(), $this->getMapper()->targetTable(), $k, DB::escape($vAnt), DB::escape($v));
						DB::queryMaster($sql);
					}
					//Comprovem si té un setter específic per actualitzar
					if (method_exists($this, 'set'.$k)) {
						call_user_func(array($this, 'set'.$k), $v);
					} else {
						$this->setField($k, $v);
					}
				}
			}
		}
		//Marquem que es tracta d'una actualització web
		$this->setField('esVerificat', 0);
		$this->setField('esWeb', 1);
		$this->setField('esPendentVerificarWeb', 1);
		$this->setField('opWeb', 'M');
		$this->setField('dataWeb', date('Y-m-d'));
		//Cridem al mapper per actualitzar
		$this->getMapper()->update($this);
	}

	function deleteWeb() {
		//fem l'eliminació real
		$this->getMapper()->delete($this);
		//Fem l'apunt des canvis al web seguiment de l'eliminació
		$ida = $this->_apuntAutoModWeb();
		//FIXME escriure log a personesCanvisWeb més elaborat
		$vAnt = DB::escape(serialize($this->dades));
		$sql = sprintf("INSERT INTO personesCanvisWeb
				(idPersona, idPersonaGrup, idK, idV, pestanya, accio, camp, valorAntic, data, esSinc, esVerificat )
				VALUES
				(%d, '%d', '%s', %d, '%s', 'E', '%s', '%s', CURDATE(), 1, 0) ", $this->getField('idPersona'), $ida, $this->getKeyName(), $this->getId(), $this->getMapper()->targetTable(), DB::escape($this->getNom()), DB::escape($vAnt));
		DB::queryMaster($sql);
	}

	function addWeb(array $dd) {
		//afegim informació creació web
		$this->setField('esVerificat', '0');
		$this->setField('esWeb', 1);
		$this->setField('opWeb', 'A');
		$this->setField('dataWeb', date('Y-m-d'));
		//els camps que ens envien hem de comprovar si hi ha un set específic per formatar les dades i si és així cal aplicar-ho
		foreach ($dd as $k=>$v) {
			if ($k!='nom' && method_exists($this, 'set'.$k)) {
				call_user_func(array($this, 'set'.$k), $v);
			}
		}
		//Cridem al mapper per actualitzar
		$id = $this->getMapper()->insert($this);
		$ida = $this->_apuntAutoModWeb();

		//FIXME escriure log a personesCanvisWeb més elaborat
		if (is_numeric($dd['idPersona'])) {
			$v = DB::escape(serialize($dd));
			$sql = sprintf("INSERT INTO personesCanvisWeb
					(idPersona, idPersonaGrup, idK, idV, pestanya, accio, camp, valor, data, esSinc, esVerificat )
					VALUES
					(%d, '%d', '%s', %d, '%s', 'A', '%s', '%s', CURDATE(), 1, 0) ", $dd['idPersona'], $ida, $this->getKeyName(), $id, $this->getMapper()->targetTable(), DB::escape($this->getNom()), DB::escape($v));
			DB::queryMaster($sql);
		}
	}

	function getId() {
		return $this->dades[$this->getKeyName()];
	}

	function setId( $id ) {
		$this->dades[$this->getKeyName()]=$id;
	}

	function getMapper() {
		$type = get_class($this).'Mapper';
		if (!class_exists($type)) {
			return null;
		}
		$m = new $type;
		return $m;
	}

	abstract function getKeyName();

}